const fs = require('fs')
const path = require('path')
const report = require('gatsby-cli/lib/reporter')
const firebase = require('firebase-admin')
const crypto = require('crypto')

const getDigest = id =>
  crypto
    .createHash('md5')
    .update(id)
    .digest('hex')

exports.sourceNodes = async ({ boundActionCreators }, { types, credential }) => {
  try {
    if (firebase.apps || !firebase.apps.length) {
      firebase.initializeApp({
        credential: firebase.credential.cert(credential),
      })
    }
  } catch (e) {
    report.warn(
      'Could not initialize Firebase. Please check `credential` property in gatsby-config.js',
    )
    report.warn(e)
    return
  }
  const db = firebase.firestore()
  db.settings({
    timestampsInSnapshots: true,
  })

  const { createNode } = boundActionCreators

  const promises = types.map(async ({ collection, type, map = node => node, write = false, dir = 'data' }) => {
    const snapshot = await db.collection(collection).get()
    for (let doc of snapshot.docs) {
      const contentDigest = getDigest(doc.id)
      const mappedData = map(doc.data())
      createNode(
        Object.assign({}, mappedData, {
          id: doc.id,
          parent: null,
          children: [],
          internal: {
            type,
            contentDigest,
          },
        }),
      )
      // if write=true we will write the data down to the folder path.
      if (write) {
        // console.log(`${type}: ${JSON.stringify(mappedData, null, 2)}`)
        if (mappedData.path) {
          const writePath = path.join(process.cwd(), `./${mappedData.path}`)
          const dirPath = path.join(process.cwd(), `./${dir}`)
          if (!fs.existsSync(dirPath)) fs.mkdirSync(dirPath, { recursive: true })
          fs.writeFileSync(writePath, mappedData.raw, { flag: 'w', encoding: 'utf8'})
        }
      }
      Promise.resolve()
    }
  })
  await Promise.all(promises)
  return
}
